package com.ruoyi.web.controller.tool;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Collections;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.context.request.async.DeferredResult;
import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 斑马订单控制器
 * 
 * @author ruoyi
 */
@Api("斑马订单接口")
@RestController
@RequestMapping("/tool/banma")
@Anonymous
public class BanmaOrderController extends BaseController {
    private static final String AUTH_TOKEN = "Bearer e5V8Vlaf9xh5i31xaI300wbdXEE3iLtgip+JXfzZsb7GShP2XCGhoVzTEVxyc8LH";
    private static final String API_URL = "https://banma365.cn/api/order/list?recipientName=&page=%d&size=%d&markFlag=0&state=4&_t=%d";
    private static final String TRACKING_URL = "https://banma365.cn/zebraExpressHub/web/tracking/getByExpressNumber/%s";
    private static final int CONNECTION_TIMEOUT = 900000000;
    private static final int READ_TIMEOUT = 900000000;
    @Autowired
    private RestTemplate restTemplate;
    public BanmaOrderController() {
            HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory();
            factory.setConnectTimeout(CONNECTION_TIMEOUT);
            factory.setReadTimeout(READ_TIMEOUT);
            restTemplate = new RestTemplate(factory);

    }

    /**
     * 获取斑马订单列表
     */
    @ApiOperation("获取斑马订单列表")
    @GetMapping("/orders")
    public R<List<Map<String, Object>>> getOrders(
            @RequestParam(value = "page", defaultValue = "1") Integer page,
            @RequestParam(value = "size", defaultValue = "10") Integer size) {
        try {
            HttpHeaders headers = new HttpHeaders();
            headers.set("Authorization", AUTH_TOKEN);
            HttpEntity<String> entity = new HttpEntity<>(headers);

            String url = String.format(API_URL, page, size, System.currentTimeMillis());
            
            ResponseEntity<Map> response = restTemplate.exchange(url, HttpMethod.GET, entity, Map.class);
            
            Map<String, Object> responseBody = response.getBody();
            if (responseBody == null) {
                return R.ok(Collections.emptyList());
            }
            
            Map<String, Object> dataMap = (Map<String, Object>) responseBody.get("data");
            if (dataMap == null) {
                return R.ok(Collections.emptyList());
            }
            
            List<Map<String, Object>> orders = (List<Map<String, Object>>) dataMap.get("list");
            
            if (orders == null) {
                return R.ok(Collections.emptyList());
            }
            
            List<Map<String, Object>> resultList = new ArrayList<>(orders.size());
            for (Map<String, Object> order : orders) {
                if (order == null) continue;
                
                Map<String, Object> simplifiedOrder = new HashMap<>();
                
                // 国际运单号和国际运费
                String trackingNumber = (String) order.get("internationalTrackingNumber");
                simplifiedOrder.put("internationalTrackingNumber", trackingNumber);
                simplifiedOrder.put("internationalShippingFee", order.get("internationalShippingFee"));
                
                // 获取物流轨迹信息
                if (trackingNumber != null && !trackingNumber.isEmpty()) {
                    simplifiedOrder.put("trackInfo", getTrackingInfo(trackingNumber));
                }
                
                // 处理子订单信息
                List<Map<String, Object>> subOrders = (List<Map<String, Object>>) order.get("subOrders");
                if (subOrders != null && !subOrders.isEmpty()) {
                    Map<String, Object> firstSubOrder = subOrders.get(0);
                    if (firstSubOrder != null) {
                        simplifiedOrder.put("poTrackingNumber", firstSubOrder.get("poTrackingNumber"));
                        simplifiedOrder.put("orderedAt", firstSubOrder.get("orderedAt"));
                        simplifiedOrder.put("timeSinceOrder", firstSubOrder.get("timeSinceOrder"));
                    }
                }
                
                resultList.add(simplifiedOrder);
            }
            
            return R.ok(resultList);
        } catch (Exception e) {
            logger.error("获取斑马订单失败", e);
            return R.fail("获取订单失败: " + e.getMessage());
        }
    }
    
    /**
     * 获取物流轨迹信息
     */
    private String getTrackingInfo(String trackingNumber) {
        try {
            if (trackingNumber == null || trackingNumber.isEmpty()) return null;
            
            String url = String.format(TRACKING_URL, trackingNumber);
            ResponseEntity<Map> response = restTemplate.getForEntity(url, Map.class);
            Map<String, Object> responseBody = response.getBody();
            
            if (responseBody == null) return null;
            
            if (Integer.valueOf(0).equals(responseBody.get("code"))) {
                Object dataObj = responseBody.get("data");
                if (dataObj == null) return null;
                
                List<Map<String, Object>> trackingData = (List<Map<String, Object>>) dataObj;
                if (trackingData != null && !trackingData.isEmpty()) {
                    Map<String, Object> firstTrack = trackingData.get(0);
                    if (firstTrack != null) {
                        return (String) firstTrack.get("track");
                    }
                }
            }
        } catch (Exception e) {
            logger.error("获取物流信息失败: {}", trackingNumber);
        }
        return null;
    }
    
    /**
     * 获取物流轨迹信息
     */
    @ApiOperation("获取物流轨迹信息")
    @GetMapping("/tracking/{trackingNumber}")
    public R<String> getTracking(@PathVariable("trackingNumber") String trackingNumber) {
        try {
            String trackInfo = getTrackingInfo(trackingNumber);
            return trackInfo != null ? R.ok(trackInfo) : R.fail("未找到物流信息");
        } catch (Exception e) {
            return R.fail("获取物流信息失败: " + e.getMessage());
        }
    }
    
    /**
     * 获取所有页的斑马订单
     */
    @ApiOperation("获取所有页的斑马订单")
    @GetMapping("/orders/all")
    public DeferredResult<R<Map<String, Object>>> getAllOrders() {
        DeferredResult<R<Map<String, Object>>> deferredResult = new DeferredResult<>(1200000L); // 20分钟超时
        
        // 异步处理请求
        Thread thread = new Thread(() -> {
            try {
                HttpHeaders headers = new HttpHeaders();
                headers.set("Authorization", AUTH_TOKEN);
                HttpEntity<String> entity = new HttpEntity<>(headers);
                
                int page = 1;
                int size = 20;  
                
                if (deferredResult.isSetOrExpired()) return;
                
                String url = String.format(API_URL, page, size, System.currentTimeMillis());
                ResponseEntity<Map> response = restTemplate.exchange(url, HttpMethod.GET, entity, Map.class);
                
                Map<String, Object> responseBody = response.getBody();
                if (responseBody == null || !responseBody.containsKey("data")) {
                    if (!deferredResult.isSetOrExpired()) {
                        deferredResult.setResult(R.fail("API返回数据无效"));
                    }
                    return;
                }
                
                Map<String, Object> dataMap = (Map<String, Object>) responseBody.get("data");
                
                // 获取总记录数
                int totalCount = dataMap.containsKey("total") ? ((Number) dataMap.get("total")).intValue() : 0;
                List<Map<String, Object>> orders = (List<Map<String, Object>>) dataMap.get("list");
                
                if (orders == null) {
                    orders = Collections.emptyList();
                }
                
                List<Map<String, Object>> allOrders = new ArrayList<>(orders.size());
                
                for (Map<String, Object> order : orders) {
                    if (order == null) continue;
                    
                    Map<String, Object> simplifiedOrder = new HashMap<>();
                    
                    // 国际运单号和国际运费
                    String trackingNumber = (String) order.get("internationalTrackingNumber");
                    simplifiedOrder.put("internationalTrackingNumber", trackingNumber);
                    simplifiedOrder.put("internationalShippingFee", order.get("internationalShippingFee"));
                    
                    // 获取物流轨迹信息
                    if (trackingNumber != null && !trackingNumber.isEmpty()) {
                        simplifiedOrder.put("trackInfo", getTrackingInfo(trackingNumber));
                    }
                    
                    // 处理子订单信息
                    List<Map<String, Object>> subOrders = (List<Map<String, Object>>) order.get("subOrders");
                    if (subOrders != null && !subOrders.isEmpty()) {
                        Map<String, Object> firstSubOrder = subOrders.get(0);
                        if (firstSubOrder != null) {
                            simplifiedOrder.put("poTrackingNumber", firstSubOrder.get("poTrackingNumber"));
                            simplifiedOrder.put("orderedAt", firstSubOrder.get("orderedAt"));
                            simplifiedOrder.put("timeSinceOrder", firstSubOrder.get("timeSinceOrder"));
                        }
                    }
                    
                    allOrders.add(simplifiedOrder);
                }
                
                // 计算预计总页数
                int totalPages = (int) Math.ceil((double) totalCount / size);
                
                Map<String, Object> resultMap = new HashMap<>();
                resultMap.put("orders", allOrders);
                resultMap.put("total", totalCount);
                resultMap.put("totalPages", totalPages);
                resultMap.put("hasMore", orders.size() == size);
                resultMap.put("nextPage", 2);
                
                // 设置结果前再次检查状态
                if (!deferredResult.isSetOrExpired()) {
                    deferredResult.setResult(R.ok(resultMap));
                }
            } catch (Exception e) {
                    deferredResult.setResult(R.fail("获取所有订单失败: " + e.getMessage()));
            }
        });
        
        thread.setDaemon(true);
        thread.start();
        
        return deferredResult;
    }
    
    /**
     * 获取下一页斑马订单
     */
    @ApiOperation("获取下一页斑马订单")
    @GetMapping("/orders/next")
    public DeferredResult<R<Map<String, Object>>> getNextPageOrders(@RequestParam(value = "page", defaultValue = "1") Integer page) {
        DeferredResult<R<Map<String, Object>>> deferredResult = new DeferredResult<>(120000L); // 2分钟超时
        
        Thread thread = new Thread(() -> {
            try {
                int size = 20; 
                // 若已超时或已完成则不处理
                if (deferredResult.isSetOrExpired()) return;
                
                R<List<Map<String, Object>>> pageResult = getOrders(page, size);
                if (pageResult.getCode() != 200) {
                    if (!deferredResult.isSetOrExpired()) {
                        deferredResult.setResult(R.fail(pageResult.getMsg()));
                    }
                    return;
                }
                
                List<Map<String, Object>> pageData = pageResult.getData();
                Map<String, Object> resultMap = new HashMap<>();
                resultMap.put("orders", pageData);
                resultMap.put("hasMore", pageData != null && pageData.size() == size);
                resultMap.put("nextPage", page + 1);
                
                // 设置结果前再次检查状态
                if (!deferredResult.isSetOrExpired()) {
                    deferredResult.setResult(R.ok(resultMap));
                }
            } catch (Exception e) {
                logger.error("获取下一页订单失败", e);
                if (!deferredResult.isSetOrExpired()) {
                    deferredResult.setResult(R.fail("获取订单失败: " + e.getMessage()));
                }
            }
        });
        
        thread.setDaemon(true);
        thread.start();
        
        return deferredResult;
    }
}   